#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "mpi.h"
#include "stdarg.h"


static int dodebug = 1;

static
int
prnt (int myrank, const char *fmt, ...) 
{
	va_list	ap;
	char    str[1024];
	char    *s;

	if (!dodebug) return 1;
	va_start (ap, fmt);
	sprintf (str, "[%d]: ", myrank);
	s = index (str, ':') + 2;
	vsnprintf (s, 1000, fmt, ap);
	s[1001] = 0;
	fputs (str, stderr);
	fflush (stderr);
	va_end (ap);
	return 1;
}



int
main (argc, argv)
	int		argc;
	char	** argv;
{
	int		myrank=-1, numprocs, mysize;
	int		i, j;
	MPI_Win			win;
	char			* base = NULL;
	int				size = 1024, numruns = 1, endebug = 1;
	int				testnum = 1;

	prnt (myrank, "MPI_Init\n");
	MPI_Init (&argc, &argv);

	for (i=1; i<argc; i++) {
		if (!strcmp (argv[i], "-n")) {
			if (++i < argc) 
				numruns = atoi (argv[i]);
		} else if (!strcmp (argv[i], "-s")) {
			if (++i < argc)
				size = atoi (argv[i]);
		} else if (!strcmp (argv[i], "-q")) {
			dodebug = 0;
			endebug = 0;
		} else if (!strcmp (argv[i], "-t")) {
			if (++i < argc)
				testnum = atoi (argv[i]);
		}
	}

	prnt (myrank, "MPI_Comm_rank\n");
	MPI_Comm_rank (MPI_COMM_WORLD, &myrank);
	prnt (myrank, "MPI_Comm_size\n");
	MPI_Comm_size (MPI_COMM_WORLD, &numprocs);

	mysize = size + (size/64) * (myrank+1);
	prnt (myrank, "MPI_Alloc_mem\n");
	MPI_Alloc_mem (mysize, MPI_INFO_NULL, &base);
	if (!base) {
		fprintf (stderr, "No shared memory\n");
		return -1; 
	}

	prnt (myrank, "MPI_Win_create\n");
	MPI_Win_create (base, mysize, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);

	prnt (myrank, "get_contig test no. 2\n");
	for (j=0; j<numruns; j++) {
		MPI_Win_fence (0, win);
		sprintf (base, "Hello, I'm Proc %d in round %d", myrank, j);
		MPI_Win_fence (0, win);

		if (myrank == 0) {
			for (i=0; i<numprocs; i++) {
				MPI_Get (	base, size, MPI_CHAR, i, 0, size, 
							MPI_CHAR, win);
				printf ("proc %d has written: %s\n", i, base);
				fflush (stdout);
			}
		}
	}
	
	prnt (myrank, "MPI_Win_fence\n");
	MPI_Win_fence (0, win);
	prnt (myrank, "MPI_Win_free\n");
	MPI_Win_free (&win);
	prnt (myrank, "MPI_Free_mem\n");
	MPI_Free_mem (base);
	prnt (myrank, "MPI_Finalize\n");
	MPI_Finalize ();
	prnt (myrank, "Good bye!\n");

	return 0;
}


























/*
 * Overrides for XEmacs and vim so that we get a uniform tabbing style.
 * XEmacs/vim will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 * vim:tw=0:ts=4:wm=0:
 */
