#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include "mpi.h"
#include "stdarg.h"
#include <sys/time.h>
#include <unistd.h>


static int dodebug = 0;

static
int
prnt (int myrank, const char *fmt, ...) 
{
	va_list	ap;
	char    str[1024];
	char    *s;

	if (!dodebug) return 1;
	va_start (ap, fmt);
	sprintf (str, "[%d]: ", myrank);
	s = index (str, ':') + 2;
	vsnprintf (s, 1000, fmt, ap);
	s[1001] = 0;
	fputs (str, stderr);
	fflush (stderr);
	va_end (ap);
	return 1;
}

static double t1;

int
micro_start ()
{
	t1 = MPI_Wtime ();
}


double
micro_get ()
{
	return (MPI_Wtime() - t1) * 1000 * 1000;
}





int
main (argc, argv)
	int		argc;
	char	** argv;
{
	int		myrank=-1, numprocs;
	int		i, j, k;
	MPI_Win			win;
	char			* base = NULL;
	char			* buffer = NULL;
	int				numruns = 1, endebug = 0;
	int				testnum = 1;
	MPI_Datatype	dtype, dtype_help;
	double			secs;
	int				mysize, minsize=64, maxsize=1024*1024;

	prnt (myrank, "MPI_Init\n");
	MPI_Init (&argc, &argv);

	for (i=1; i<argc; i++) {
		if (!strcmp (argv[i], "-n")) {
			if (++i < argc) 
				numruns = atoi (argv[i]);
		} else if (!strcmp (argv[i], "-s")) {
			if (++i < argc)
				maxsize = atoi (argv[i]);
		} else if (!strcmp (argv[i], "-m")) {
			if (++i < argc)
				minsize = atoi (argv[i]);
		} else if (!strcmp (argv[i], "-q")) {
			dodebug = 0;
			endebug = 0;
		} else if (!strcmp (argv[i], "-d")) {
			dodebug = 1;
			endebug = 1;
		}
	}

	prnt (myrank, "MPI_Comm_rank\n");
	MPI_Comm_rank (MPI_COMM_WORLD, &myrank);
	prnt (myrank, "MPI_Comm_size\n");
	MPI_Comm_size (MPI_COMM_WORLD, &numprocs);

	prnt (myrank, "MPI_Alloc_mem\n");
	MPI_Alloc_mem (maxsize, MPI_INFO_NULL, &base);
	if (!base) {
		fprintf (stderr, "No shared memory\n");
		return -1; 
	}
	buffer = malloc (maxsize);
	if (!buffer) {
		fprintf (stderr, "Out of memory\n");
		return -2;
	}

	prnt (myrank, "MPI_Win_create\n");
	MPI_Win_create (base, maxsize, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);

	prnt (myrank, "put_contig test no. 1\n");
	MPI_Win_fence (0, win);
	for (mysize=minsize; mysize <= maxsize; mysize = mysize <= 256 ? 
											mysize +4 : mysize <= 1024 ? 
											mysize + 16 : (mysize*3)/2) {
		if (myrank == 1) {
			micro_start ();
			for (j=0; j<numruns; j++) {
				MPI_Send (buffer, mysize, MPI_CHAR, 0, 0, MPI_COMM_WORLD);
			}
			secs = micro_get();

			printf ("%d %8.3lf\n", mysize, secs / (double)numruns);
			fflush (stdin);
		} else if (myrank == 0) {
			for (j=0; j<numruns; j++) {
				MPI_Recv (	buffer, mysize, MPI_CHAR, 1, 0, 
							MPI_COMM_WORLD, NULL);
			}
		}
	}
	
	prnt (myrank, "MPI_Win_fence\n");
	MPI_Win_fence (0, win);
	prnt (myrank, "MPI_Win_free\n");
	MPI_Win_free (&win);
	prnt (myrank, "MPI_Finalize\n");
	MPI_Finalize ();
	prnt (myrank, "Good bye!\n");

	return 0;
}


























/*
 * Overrides for XEmacs and vim so that we get a uniform tabbing style.
 * XEmacs/vim will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 * vim:tw=0:ts=4:wm=0:
 */
