#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>

#include "mpi.h"

extern char *optarg;

static int dodebug = 0;

static
int
prnt (int myrank, const char *fmt, ...) 
{
	va_list	ap;
	char    str[1024];
	char    *s;

	if (!dodebug) return 1;
	va_start (ap, fmt);
	sprintf (str, "[%d]: ", myrank);
	s = index (str, ':') + 2;
	vsnprintf (s, 1000, fmt, ap);
	s[1001] = 0;
	fputs (str, stderr);
	fflush (stderr);
	va_end (ap);
	return 1;
}



int
main (argc, argv)
	int		argc;
	char	** argv;
{
	int		myrank=-1, numprocs, mysize;
	int		i, j, c;
	MPI_Win			win;
	char			* base = NULL;
	char			buffer [2048];
	int				size = 1024, numruns = 1, endebug = 1;
	int				testnum = 1;
	MPI_Datatype	dtype, dtype_help;

	MPI_Init (&argc, &argv);
	MPI_Comm_rank (MPI_COMM_WORLD, &myrank);
	MPI_Comm_size (MPI_COMM_WORLD, &numprocs);

    while((c = getopt(argc, argv, "n:s:t:v?")) != EOF) {
		switch(c) {
		case 'n':
			numruns = atoi (optarg);
			break;
		case 's':
			size = atoi (optarg);
			break;
		case 'v':
			dodebug = 1;
			endebug = 1;
			break;
		case 't':
			testnum = atoi (optarg);
			break;
		case '?':
			if (!myrank)
				printf ("Test the functionality of MPI_Put(). Options:\n\
 -n value\t\tnumber of full testloops\n\
 -s value\t\tsize of data block to get\n\
 -t value\t\tnumber of data blocks to get\n\
 -v\t\tbe verbose\n");
			MPI_Finalize();
			break;
		}
	}

	mysize = size + (size/64) * (myrank+1);
	prnt (myrank, "MPI_Alloc_mem\n");
	MPI_Alloc_mem (mysize, MPI_INFO_NULL, &base);
	if (!base) {
		fprintf (stderr, "No shared memory\n");
		MPI_Abort (MPI_COMM_WORLD, -1); 
	}

	prnt (myrank, "MPI_Win_create\n");
	MPI_Win_create (base, mysize, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);

#define LOCK 1

	prnt (myrank, "put_contig test no. 1\n");
	for (j=0; j<numruns; j++) {
		prnt (myrank, "MPI_Win_fence (1)\n");
		MPI_Win_fence (0, win);
		sprintf (buffer, "Hello, I'm Proc %d in round %d", myrank, j);
		prnt (myrank, "MPI_Win_fence (2)\n");
		MPI_Win_fence (0, win);
#if LOCK
		prnt (myrank, "MPI_Win_lock \n");
		MPI_Win_lock (MPI_LOCK_EXCLUSIVE, 0, 0, win);
#endif
		prnt (myrank, "MPI_Put \n");
		MPI_Put (buffer, 256, MPI_CHAR, 0, 256 * myrank, 256, MPI_CHAR, win); 
#if LOCK
		prnt (myrank, "MPI_Win_unlock \n");
		MPI_Win_unlock (0, win);
#endif
		prnt (myrank, "MPI_Win_fence (3)\n");
		MPI_Win_fence (0, win);

		if (myrank == 0) {
			for (i=0; i<numprocs; i++) {
				printf ("proc %d has written: %s\n", i, base + 256*i);
				fflush (stdout);
			}
		}
	}
	
	prnt (myrank, "MPI_Win_fence\n");
	MPI_Win_fence (0, win);
	prnt (myrank, "MPI_Win_free\n");
	MPI_Win_free (&win);
	prnt (myrank, "MPI_Finalize\n");
	MPI_Finalize ();
	prnt (myrank, "Good bye!\n");

	return 0;
}


























/*
 * Overrides for XEmacs and vim so that we get a uniform tabbing style.
 * XEmacs/vim will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 * vim:tw=0:ts=4:wm=0:
 */
