#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>
#include <sys/time.h>
#include <unistd.h>

#include "mpi.h"

extern char *optarg;

static int dodebug = 0;

static
int
prnt (int myrank, const char *fmt, ...) 
{
	va_list	ap;
	char    str[1024];
	char    *s;

	if (!dodebug) return 1;
	va_start (ap, fmt);
	sprintf (str, "[%d]: ", myrank);
	s = index (str, ':') + 2;
	vsnprintf (s, 1000, fmt, ap);
	s[1001] = 0;
	fputs (str, stderr);
	fflush (stderr);
	va_end (ap);
	return 1;
}


int
main (argc, argv)
	int		argc;
	char	** argv;
{
	int		myrank=-1, numprocs;
	int		i, j, k, c;
	MPI_Win			win;
	char			* base = NULL;
	char			* buffer = NULL;
	int				numruns = 1, endebug = 0;
	int				testnum = 1;
	MPI_Datatype	dtype, dtype_help;
	double			wtime;
	int				mysize, minsize=64, maxsize=1024*1024;

	MPI_Init (&argc, &argv);
	MPI_Comm_rank (MPI_COMM_WORLD, &myrank);
	MPI_Comm_size (MPI_COMM_WORLD, &numprocs);

    while((c = getopt(argc, argv, "n:m:s:t:v?")) != EOF) {
		switch(c) {
		case 'n':
			numruns = atoi (optarg);
			break;
		case 's':
			maxsize = atoi (optarg);
			break;
		case 'v':
			dodebug = 1;
			endebug = 1;
			break;
		case 'm':
			minsize = atoi (optarg);
			break;
		case 't':
			testnum = atoi (optarg);
			break;
		case '?':
			if (!myrank)
				printf ("Test the functionality of MPI_Put(). Options:\n\
 -n value\t\tnumber of full testloops\n\
 -s value\t\tmax. size of data block to get\n\
 -m value\t\tmin. size of data block to get\n\
 -t value\t\tnumber of data blocks to get\n\
 -v\t\tbe verbose\n");
			MPI_Finalize();
			break;
		}
	}

	prnt (myrank, "MPI_Alloc_mem\n");
	MPI_Alloc_mem (maxsize, MPI_INFO_NULL, &base);
	if (!base) {
		fprintf (stderr, "No shared memory\n");
		return -1; 
	}
	buffer = malloc (maxsize);
	if (!buffer) {
		fprintf (stderr, "Out of memory\n");
		return -2;
	}

	prnt (myrank, "MPI_Win_create\n");
	MPI_Win_create (base, maxsize, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);

	prnt (myrank, "put_contig test\n");
	if (myrank == 1)
		printf ("#size\tlatency [ms]\n");
	MPI_Win_fence (0, win);
	for (mysize=minsize; mysize <= maxsize; mysize = mysize <= 256 ? 
											mysize +4 : mysize <= 1024 ? 
											mysize + 16 : (mysize*3)/2) {
		if (myrank == 1) {
			wtime = MPI_Wtime();
			for (j=0; j<numruns; j++) {
				MPI_Put (	buffer, mysize, MPI_CHAR, 0, 0, 
							mysize, MPI_CHAR, win); 
			}
			wtime = MPI_Wtime() - wtime;;

			printf ("%d %8.3lf\n", mysize, (wtime*1000) / (double)numruns);
			fflush (stdout);
		}
	}
	
	prnt (myrank, "MPI_Win_fence\n");
	MPI_Win_fence (0, win);
	prnt (myrank, "MPI_Win_free\n");
	MPI_Win_free (&win);
	MPI_Finalize ();
	prnt (myrank, "Good bye!\n");

	return 0;
}


























/*
 * Overrides for XEmacs and vim so that we get a uniform tabbing style.
 * XEmacs/vim will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 * vim:tw=0:ts=4:wm=0:
 */
